package sun.java2d.windows;

import sun.java2d.loops.GraphicsPrimitive;
import sun.java2d.pipe.ShapeSpanIterator;
import sun.java2d.pipe.LoopPipe;
import sun.java2d.pipe.SpanIterator;
import java.awt.Shape;
import java.awt.geom.Path2D;
import sun.java2d.InvalidPipeException;
import sun.java2d.SunGraphics2D;
import java.awt.Composite;
import sun.java2d.pipe.Region;
import sun.java2d.pipe.ShapeDrawPipe;
import sun.java2d.pipe.PixelFillPipe;
import sun.java2d.pipe.PixelDrawPipe;

public class GDIRenderer implements PixelDrawPipe, PixelFillPipe, ShapeDrawPipe
{
    public GDIRenderer() {
    }
    
    native void doDrawLine(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7);
    
    @Override
    public void drawLine(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        int transX = sunGraphics2D.transX;
        int transY = sunGraphics2D.transY;
        try {
            this.doDrawLine((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + transX, n2 + transY, n3 + transX, n4 + transY);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doDrawRect(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7);
    
    @Override
    public void drawRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        try {
            this.doDrawRect((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doDrawRoundRect(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7, int p8, int p9);
    
    @Override
    public void drawRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.doDrawRoundRect((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4, n5, n6);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doDrawOval(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7);
    
    @Override
    public void drawOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        try {
            this.doDrawOval((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doDrawArc(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7, int p8, int p9);
    
    @Override
    public void drawArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.doDrawArc((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4, n5, n6);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doDrawPoly(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int[] p6, int[] p7, int p8, boolean p9);
    
    @Override
    public void drawPolyline(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        try {
            this.doDrawPoly((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, sunGraphics2D.transX, sunGraphics2D.transY, array, array2, n, false);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    @Override
    public void drawPolygon(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        try {
            this.doDrawPoly((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, sunGraphics2D.transX, sunGraphics2D.transY, array, array2, n, true);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doFillRect(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7);
    
    @Override
    public void fillRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        try {
            this.doFillRect((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doFillRoundRect(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7, int p8, int p9);
    
    @Override
    public void fillRoundRect(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.doFillRoundRect((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4, n5, n6);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doFillOval(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7);
    
    @Override
    public void fillOval(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4) {
        try {
            this.doFillOval((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doFillArc(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int p6, int p7, int p8, int p9);
    
    @Override
    public void fillArc(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.doFillArc((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, n + sunGraphics2D.transX, n2 + sunGraphics2D.transY, n3, n4, n5, n6);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doFillPoly(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, int[] p6, int[] p7, int p8);
    
    @Override
    public void fillPolygon(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n) {
        try {
            this.doFillPoly((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, sunGraphics2D.transX, sunGraphics2D.transY, array, array2, n);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    native void doShape(GDIWindowSurfaceData p0, Region p1, Composite p2, int p3, int p4, int p5, Path2D.Float p6, boolean p7);
    
    void doShape(SunGraphics2D sunGraphics2D, Shape shape, boolean b) {
        Path2D.Float float1;
        int transX;
        int transY;
        if (sunGraphics2D.transformState <= 1) {
            if ((shape instanceof Path2D.Float)) {
                float1 = (Path2D.Float)shape;
            }
            else {
                float1 = new Path2D.Float(shape);
            }
            transX = sunGraphics2D.transX;
            transY = sunGraphics2D.transY;
        }
        else {
            float1 = new Path2D.Float(shape, sunGraphics2D.transform);
            transX = 0;
            transY = 0;
        }
        try {
            this.doShape((GDIWindowSurfaceData)sunGraphics2D.surfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, sunGraphics2D.eargb, transX, transY, float1, b);
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
    }
    
    public void doFillSpans(SunGraphics2D sunGraphics2D, SpanIterator spanIterator) {
        int[] array = new int[4];
        GDIWindowSurfaceData gdiWindowSurfaceData;
        try {
            gdiWindowSurfaceData = (GDIWindowSurfaceData)sunGraphics2D.surfaceData;
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
        }
        Region compClip = sunGraphics2D.getCompClip();
        Composite composite = sunGraphics2D.composite;
        int eargb = sunGraphics2D.eargb;
        while (spanIterator.nextSpan(array)) {
            this.doFillRect(gdiWindowSurfaceData, compClip, composite, eargb, array[0], array[1], array[2] - array[0], array[3] - array[1]);
        }
    }
    
    @Override
    public void draw(SunGraphics2D sg2d, Shape s) {
        if (sg2d.strokeState == 0) {
            this.doShape(sg2d, s, false);
        }
        else if (sg2d.strokeState < 3) {
            ShapeSpanIterator strokeSpans = LoopPipe.getStrokeSpans(sg2d, s);
            try {
                this.doFillSpans(sg2d, strokeSpans);
            }
            finally {
                strokeSpans.dispose();
            }
        }
        else {
            this.doShape(sg2d, sg2d.stroke.createStrokedShape(s), true);
        }
    }
    
    @Override
    public void fill(SunGraphics2D sunGraphics2D, Shape shape) {
        this.doShape(sunGraphics2D, shape, true);
    }
    
    public native void devCopyArea(GDIWindowSurfaceData p0, int p1, int p2, int p3, int p4, int p5, int p6);
    
    public GDIRenderer traceWrap() {
        return new GDIRenderer.Tracer();
    }
    
    public static class Tracer extends GDIRenderer
    {
        @Override
        void doDrawLine(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5) {
            GraphicsPrimitive.tracePrimitive("GDIDrawLine");
            super.doDrawLine(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5);
        }
        
        @Override
        void doDrawRect(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5) {
            GraphicsPrimitive.tracePrimitive("GDIDrawRect");
            super.doDrawRect(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5);
        }
        
        @Override
        void doDrawRoundRect(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
            GraphicsPrimitive.tracePrimitive("GDIDrawRoundRect");
            super.doDrawRoundRect(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5, n6, n7);
        }
        
        @Override
        void doDrawOval(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5) {
            GraphicsPrimitive.tracePrimitive("GDIDrawOval");
            super.doDrawOval(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5);
        }
        
        @Override
        void doDrawArc(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
            GraphicsPrimitive.tracePrimitive("GDIDrawArc");
            super.doDrawArc(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5, n6, n7);
        }
        
        @Override
        void doDrawPoly(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int[] array, int[] array2, int n4, boolean b) {
            GraphicsPrimitive.tracePrimitive("GDIDrawPoly");
            super.doDrawPoly(gdiWindowSurfaceData, region, composite, n, n2, n3, array, array2, n4, b);
        }
        
        @Override
        void doFillRect(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5) {
            GraphicsPrimitive.tracePrimitive("GDIFillRect");
            super.doFillRect(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5);
        }
        
        @Override
        void doFillRoundRect(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
            GraphicsPrimitive.tracePrimitive("GDIFillRoundRect");
            super.doFillRoundRect(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5, n6, n7);
        }
        
        @Override
        void doFillOval(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5) {
            GraphicsPrimitive.tracePrimitive("GDIFillOval");
            super.doFillOval(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5);
        }
        
        @Override
        void doFillArc(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int n4, int n5, int n6, int n7) {
            GraphicsPrimitive.tracePrimitive("GDIFillArc");
            super.doFillArc(gdiWindowSurfaceData, region, composite, n, n2, n3, n4, n5, n6, n7);
        }
        
        @Override
        void doFillPoly(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, int[] array, int[] array2, int n4) {
            GraphicsPrimitive.tracePrimitive("GDIFillPoly");
            super.doFillPoly(gdiWindowSurfaceData, region, composite, n, n2, n3, array, array2, n4);
        }
        
        @Override
        void doShape(GDIWindowSurfaceData gdiWindowSurfaceData, Region region, Composite composite, int n, int n2, int n3, Path2D.Float float1, boolean b) {
            GraphicsPrimitive.tracePrimitive(b ? "GDIFillShape" : "GDIDrawShape");
            super.doShape(gdiWindowSurfaceData, region, composite, n, n2, n3, float1, b);
        }
        
        @Override
        public void devCopyArea(GDIWindowSurfaceData gdiWindowSurfaceData, int n, int n2, int n3, int n4, int n5, int n6) {
            GraphicsPrimitive.tracePrimitive("GDICopyArea");
            super.devCopyArea(gdiWindowSurfaceData, n, n2, n3, n4, n5, n6);
        }
    }
}
